package sb.finder;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileFilter;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JRadioButton;
import javax.swing.JCheckBox;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import olivamail.core.common.IOUtils;
import java.awt.Dimension;
import java.awt.CardLayout;

public class MainFrame extends JFrame {

	static final long serialVersionUID = 6229857014381667410L;
	private JPanel jContentPane = null;
	private JTabbedPane jTabbedPane1 = null;
	private JPanel tabMain = null;
	private JPanel tabSettings = null;
	private JTextField jtfSourceDir = null;
	private JButton jbSourceDir = null;
	private JTextField jtfMinSize = null;
	private JComboBox jcbMinSize = null;
	private JTextField jtfMaxSize = null;
	private JComboBox jcbMaxSize = null;
	private JRadioButton jrbTargetSubfolder = null;
	private JRadioButton jrbRootSubfolder = null;
	private JRadioButton jrbSpecifedFolder = null;
	private JCheckBox jcbOnlyUsFiles = null;
	private JRadioButton jrbIterateYes = null;
	private JRadioButton jrbIterateSizeDiffers = null;
	private JRadioButton jrbIterateCtntDiffers = null;
	private JRadioButton jrbIterateNoSkip = null;
	private JRadioButton jrbIterateNoRewrite = null;
	private JCheckBox jcbClearTargetFolder = null;
	private JCheckBox jcbSearchByCriteria = null;
	private JTextField jtfSearchCriteria = null;
	private JProgressBar progressBar1 = null;
	private JButton jbGo = null;
	private JScrollPane jScrollPane1 = null;
	private JEditorPane jEditorPane1 = null;
	
	private static final String sizeArray[] = { "bytes", "Kb", "Mb" };
	private static final String extArray[] = { ".jpg", ".jpeg", ".png", ".bmp", ".gif", ".tiff" };
	private static final String SUBFOLDER = "__PICS";
	
	private JLabel jlMinSize = null;
	private JLabel jlMaxSize = null;
	private JTextField jtfSpecifiedFolder = null;
	private JButton jbSpecFolder = null;
	private JPanel rightPanel = null;
	private JLabel imageLabel = null;
	private JCheckBox jcbPreview = null;
	private JButton stopButton = null;
	private JLabel jlGeometry = null;
	private JTextField jtfMinGeometry = null;
	
	private int geometry1 = 300;
	private int geometry2 = 200;
	
	/**
	 * This is the default constructor
	 */
	public MainFrame() {
		super();
		initialize();
		try {
			loadSettings();
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(750, 512);
		this.setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
		this.setResizable(false);
		this.setTitle(Strings.getString("MainFrame.title"));
		this.setContentPane(getJContentPane());
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJTabbedPane1(), java.awt.BorderLayout.CENTER);
			jContentPane.add(getRightPanel(), java.awt.BorderLayout.EAST);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jTabbedPane1	
	 * 	
	 * @return javax.swing.JTabbedPane	
	 */    
	private JTabbedPane getJTabbedPane1() {
		if (jTabbedPane1 == null) {
			jTabbedPane1 = new JTabbedPane();
			jTabbedPane1.addTab(Strings.getString("MainFrame.tabMain"), null, getTabMain(), null);
			jTabbedPane1.addTab(Strings.getString("MainFrame.tabSettings"), null, getTabSettings(), null);
		}
		return jTabbedPane1;
	}

	/**
	 * This method initializes tabMain	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getTabMain() {
		if (tabMain == null) {
			jlMaxSize = new JLabel();
			jlMaxSize.setText(Strings.getString("MainFrame.maxSize"));
			jlMinSize = new JLabel();
			jlMinSize.setText(Strings.getString("MainFrame.minSize"));
			tabMain = new JPanel();
			tabMain.add(getJtfSourceDir(), null);
			tabMain.add(getJbSourceDir(), null);
			tabMain.add(jlMinSize, null);
			tabMain.add(getJtfMinSize(), null);
			tabMain.add(getJcbMinSize(), null);
			tabMain.add(jlMaxSize, null);
			tabMain.add(getJtfMaxSize(), null);
			tabMain.add(getJcbMaxSize(), null);
			tabMain.add(getJrbTargetSubfolder(), null);
			tabMain.add(getJrbRootSubfolder(), null);
			tabMain.add(getJrbSpecifedFolder(), null);
			tabMain.add(getJtfSpecifiedFolder(), null);
			tabMain.add(getJbSpecFolder(), null);
			tabMain.add(getJcbOnlyUsFiles(), null);
			tabMain.add(getJrbIterateYes(), null);
			tabMain.add(getJrbIterateSizeDiffers(), null);
			tabMain.add(getJrbIterateCtntDiffers(), null);
			tabMain.add(getJrbIterateNoSkip(), null);
			tabMain.add(getJrbIterateNoRewrite(), null);
			tabMain.add(getJcbClearTargetFolder(), null);
			tabMain.add(getJcbPreview(), null);
			tabMain.add(getJcbSearchByCriteria(), null);
			tabMain.add(getJtfSearchCriteria(), null);
			tabMain.add(getProgressBar1(), null);
			tabMain.add(getStopButton(), null);
			tabMain.add(getJbGo(), null);
			tabMain.add(getJScrollPane1(), null);
			ButtonGroup tbg = new ButtonGroup();
			tbg.add(jrbTargetSubfolder);
			tbg.add(jrbRootSubfolder);
			tbg.add(jrbSpecifedFolder);
			ButtonGroup ibg = new ButtonGroup();
			ibg.add(jrbIterateYes);
			ibg.add(jrbIterateSizeDiffers);
			ibg.add(jrbIterateCtntDiffers);
			ibg.add(jrbIterateNoSkip);
			ibg.add(jrbIterateNoRewrite);
		}
		return tabMain;
	}

	/**
	 * This method initializes tabSettings	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getTabSettings() {
		if (tabSettings == null) {
			jlGeometry = new JLabel();
			jlGeometry.setText(Strings.getString("MainFrame.minGeometry"));
			jlGeometry.setPreferredSize(new java.awt.Dimension(240,16));
			tabSettings = new JPanel();
			tabSettings.add(jlGeometry, null);
			tabSettings.add(getJtfMinGeometry(), null);
		}
		return tabSettings;
	}

	/**
	 * This method initializes jtfSourceDir	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getJtfSourceDir() {
		if (jtfSourceDir == null) {
			jtfSourceDir = new JTextField();
			jtfSourceDir.setPreferredSize(new java.awt.Dimension(400,20));
			TextPopup.registerComponent(jtfSourceDir);
		}
		return jtfSourceDir;
	}

	/**
	 * This method initializes jbSourceDir	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJbSourceDir() {
		if (jbSourceDir == null) {
			jbSourceDir = new JButton();
			jbSourceDir.setText("...");
			jbSourceDir.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jbSourceDir_actionPerformed(e);
				}
			});
			jbSourceDir.setPreferredSize(new java.awt.Dimension(43,20));
		}
		return jbSourceDir;
	}

	void jbSourceDir_actionPerformed(java.awt.event.ActionEvent e) {
		File file = Context.dirOpenDialog(this);
		if(file == null)
			return;
		jtfSourceDir.setText(file.getPath());			
	}
	
	/**
	 * This method initializes jtfMinSize	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getJtfMinSize() {
		if (jtfMinSize == null) {
			jtfMinSize = new JTextField();
			jtfMinSize.setPreferredSize(new java.awt.Dimension(60,20));
			jtfMinSize.setText("10");
			TextPopup.registerComponent(jtfMinSize);
		}
		return jtfMinSize;
	}

	/**
	 * This method initializes jcbMinSize	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJcbMinSize() {
		if (jcbMinSize == null) {
			jcbMinSize = new JComboBox(sizeArray);
			jcbMinSize.setPreferredSize(new java.awt.Dimension(60,20));
			jcbMinSize.setSelectedIndex(1);			
		}
		return jcbMinSize;
	}

	/**
	 * This method initializes jtfMaxSize	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getJtfMaxSize() {
		if (jtfMaxSize == null) {
			jtfMaxSize = new JTextField();
			jtfMaxSize.setPreferredSize(new java.awt.Dimension(60,20));
			jtfMaxSize.setEnabled(false);
			TextPopup.registerComponent(jtfMaxSize);
		}
		return jtfMaxSize;
	}

	/**
	 * This method initializes jcbMaxSize	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJcbMaxSize() {
		if (jcbMaxSize == null) {
			jcbMaxSize = new JComboBox(sizeArray);
			jcbMaxSize.setPreferredSize(new java.awt.Dimension(60,20));
			jcbMaxSize.setEnabled(false);
			jcbMaxSize.setSelectedIndex(2);
		}
		return jcbMaxSize;
	}

	/**
	 * This method initializes jrbTargetSubfolder	
	 * 	
	 * @return javax.swing.JRadioButton	
	 */    
	private JRadioButton getJrbTargetSubfolder() {
		if (jrbTargetSubfolder == null) {
			jrbTargetSubfolder = new JRadioButton();
			jrbTargetSubfolder.setPreferredSize(new java.awt.Dimension(450,20));
			jrbTargetSubfolder.setSelected(true);
			jrbTargetSubfolder.setText(Strings.getString("MainFrame.targetSubfolder"));
		}
		return jrbTargetSubfolder;
	}

	/**
	 * This method initializes jrbRootSubfolder	
	 * 	
	 * @return javax.swing.JRadioButton	
	 */    
	private JRadioButton getJrbRootSubfolder() {
		if (jrbRootSubfolder == null) {
			jrbRootSubfolder = new JRadioButton();
			jrbRootSubfolder.setPreferredSize(new java.awt.Dimension(450,20));
			jrbRootSubfolder.setText(Strings.getString("MainFrame.rootSubfolder"));
		}
		return jrbRootSubfolder;
	}

	/**
	 * This method initializes jcbSpecifedFolder	
	 * 	
	 * @return javax.swing.JRadioButton	
	 */    
	private JRadioButton getJrbSpecifedFolder() {
		if (jrbSpecifedFolder == null) {
			jrbSpecifedFolder = new JRadioButton();
			jrbSpecifedFolder.setPreferredSize(new java.awt.Dimension(160,20));
			jrbSpecifedFolder.setText(Strings.getString("MainFrame.specifiedFolder"));
			jrbSpecifedFolder.addChangeListener(new javax.swing.event.ChangeListener() { 
				public void stateChanged(javax.swing.event.ChangeEvent e) {    
					jtfSpecifiedFolder.setEnabled(jrbSpecifedFolder.isSelected());
				}
			});
		}
		return jrbSpecifedFolder;
	}

	/**
	 * This method initializes jcbOnlyUsFiles	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */    
	private JCheckBox getJcbOnlyUsFiles() {
		if (jcbOnlyUsFiles == null) {
			jcbOnlyUsFiles = new JCheckBox();
			jcbOnlyUsFiles.setPreferredSize(new java.awt.Dimension(450,20));
			jcbOnlyUsFiles.setText((String)Strings.getString("MainFrame.onlyUsFiles"));
		}
		return jcbOnlyUsFiles;
	}

	/**
	 * This method initializes jcbIterateYes	
	 * 	
	 * @return javax.swing.JRadioButton	
	 */    
	private JRadioButton getJrbIterateYes() {
		if (jrbIterateYes == null) {
			jrbIterateYes = new JRadioButton();
			jrbIterateYes.setPreferredSize(new java.awt.Dimension(450,20));
			jrbIterateYes.setText(Strings.getString("MainFrame.iterateYes"));	
		}
		return jrbIterateYes;
	}

	/**
	 * This method initializes jcbIterateSizeDiffer	
	 * 	
	 * @return javax.swing.JRadioButton	
	 */    
	private JRadioButton getJrbIterateSizeDiffers() {
		if (jrbIterateSizeDiffers == null) {
			jrbIterateSizeDiffers = new JRadioButton();
			jrbIterateSizeDiffers.setPreferredSize(new java.awt.Dimension(450,20));
			jrbIterateSizeDiffers.setSelected(false);
			jrbIterateSizeDiffers.setText(Strings.getString("MainFrame.iterateSizeDiffers"));
		}
		return jrbIterateSizeDiffers;
	}

	/**
	 * This method initializes jcbIterateCtntDiffers	
	 * 	
	 * @return javax.swing.JRadioButton	
	 */    
	private JRadioButton getJrbIterateCtntDiffers() {
		if (jrbIterateCtntDiffers == null) {
			jrbIterateCtntDiffers = new JRadioButton();
			jrbIterateCtntDiffers.setPreferredSize(new java.awt.Dimension(450,20));
			jrbIterateCtntDiffers.setSelected(true);
			jrbIterateCtntDiffers.setText(Strings.getString("MainFrame.iterateCtntDiffers"));
		}
		return jrbIterateCtntDiffers;
	}

	/**
	 * This method initializes jcbIterateNoSkip	
	 * 	
	 * @return javax.swing.JRadioButton	
	 */    
	private JRadioButton getJrbIterateNoSkip() {
		if (jrbIterateNoSkip == null) {
			jrbIterateNoSkip = new JRadioButton();
			jrbIterateNoSkip.setPreferredSize(new java.awt.Dimension(450,20));
			jrbIterateNoSkip.setText(Strings.getString("MainFrame.iterateNoSkip"));
		}
		return jrbIterateNoSkip;
	}

	/**
	 * This method initializes jcbIterateNoRewrite	
	 * 	
	 * @return javax.swing.JRadioButton	
	 */    
	private JRadioButton getJrbIterateNoRewrite() {
		if (jrbIterateNoRewrite == null) {
			jrbIterateNoRewrite = new JRadioButton();
			jrbIterateNoRewrite.setPreferredSize(new java.awt.Dimension(450,20));
			jrbIterateNoRewrite.setText(Strings.getString("MainFrame.iterateNoRewrite"));
		}
		return jrbIterateNoRewrite;
	}

	/**
	 * This method initializes jcbClearTargetFolder	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */    
	private JCheckBox getJcbClearTargetFolder() {
		if (jcbClearTargetFolder == null) {
			jcbClearTargetFolder = new JCheckBox();
			jcbClearTargetFolder.setPreferredSize(new java.awt.Dimension(450,20));
			jcbClearTargetFolder.setText((String)Strings.getString("MainFrame.clearTargetFolder"));
		}
		return jcbClearTargetFolder;
	}

	/**
	 * This method initializes jcbSearchByCriteria	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */    
	private JCheckBox getJcbSearchByCriteria() {
		if (jcbSearchByCriteria == null) {
			jcbSearchByCriteria = new JCheckBox();
			jcbSearchByCriteria.setPreferredSize(new java.awt.Dimension(220,20));
			jcbSearchByCriteria.setEnabled(false);
			jcbSearchByCriteria.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jtfSearchCriteria.setEnabled(jcbSearchByCriteria.isSelected());
				}
			});
			jcbSearchByCriteria.setText((String)Strings.getString("MainFrame.searchByCriteria"));
		}
		return jcbSearchByCriteria;
	}

	/**
	 * This method initializes jtfSearchCriteria	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getJtfSearchCriteria() {
		if (jtfSearchCriteria == null) {
			jtfSearchCriteria = new JTextField();
			jtfSearchCriteria.setPreferredSize(new java.awt.Dimension(220,20));
			jtfSearchCriteria.setEnabled(false);
			TextPopup.registerComponent(jtfSearchCriteria);
		}
		return jtfSearchCriteria;
	}

	/**
	 * This method initializes progressBar1	
	 * 	
	 * @return javax.swing.JProgressBar	
	 */    
	private JProgressBar getProgressBar1() {
		if (progressBar1 == null) {
			progressBar1 = new JProgressBar();
			progressBar1.setPreferredSize(new java.awt.Dimension(280,20));
		}
		return progressBar1;
	}

	/**
	 * This method initializes jbGo	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJbGo() {
		if (jbGo == null) {
			jbGo = new JButton();
			jbGo.setText(Strings.getString("MainFrame.findButton"));
			jbGo.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {					
					String fname = jtfSourceDir.getText().trim();
					if(fname.equals("")) {
						AlertDialog.showWarningDialog(Strings.getString("MainFrame.warnSpecSource"));
						return;
					}
					
					try {
						applySettings();
					} catch (Exception exc) {
						exc.printStackTrace();
						AlertDialog.showErrorDialog(exc);
					}
					
					if(jcbPreview.isSelected()) {
						imageLabel.setText("");
						imageLabel.setIcon(null);
					}
					jEditorPane1.setText("");
					stopButton.setEnabled(true);
					
					final SwingWorker worker = new SwingWorker() {
						public Object construct() {
							collectFiles();
							return null;
						}
					};
					worker.start();
				}
			});
			jbGo.setPreferredSize(new java.awt.Dimension(90, 20));
		}
		return jbGo;
	}

	/**
	 * This method initializes jScrollPane1
	 * 
	 * @return javax.swing.JScrollPane
	 */    
	private JScrollPane getJScrollPane1() {
		if (jScrollPane1 == null) {
			jScrollPane1 = new JScrollPane();
			jScrollPane1.setPreferredSize(new java.awt.Dimension(445,70));
			jScrollPane1.setViewportView(getJEditorPane1());
		}
		return jScrollPane1;
	}

	/**
	 * This method initializes jEditorPane1	
	 * 	
	 * @return javax.swing.JEditorPane	
	 */    
	private JEditorPane getJEditorPane1() {
		if (jEditorPane1 == null) {
			jEditorPane1 = new JEditorPane();
			TextPopup.registerComponent(jEditorPane1);
		}
		return jEditorPane1;
	}

	private void collectFiles() {
		File dir = new File(jtfSourceDir.getText().trim());
		Cursor oldCursor = getCursor();
		try {
			setCursor(new Cursor(Cursor.WAIT_CURSOR));
			
			String minGeo = jtfMinGeometry.getText().trim().toLowerCase();
			jtfMinGeometry.setText(minGeo);
			
			if(minGeo.matches("\\d+x\\d+")) {			
				int idx = minGeo.indexOf('x');
				geometry1 = Integer.parseInt(minGeo.substring(0, idx));
				geometry2 = Integer.parseInt(minGeo.substring(idx + 1));
				System.out.println(Context.myName + ": " + "Geometry " + 
						geometry1 + "x" + geometry2 + " will be used.");				
			} else {
				System.out.println(Context.myName + ": " + "Min geometry does not correct. The default " + 
						geometry1 + "x" + geometry2 + " will be used.");
			}
			
			if(!dir.exists())
				AlertDialog.showWarningDialog(Strings.getString("MainFrame.warnSourceNotExists"));
			else
				processDirectory(dir, null, true);
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		} finally {
			setCursor(oldCursor);			
		}
	}

	private void processDirectory(File dir, Subfolder subfolder, boolean progressIt) throws Exception {
		if (subfolder == null) {
			if (jrbSpecifedFolder.isSelected()) {
				subfolder = new Subfolder(dir);
				subfolder.dir = new File(jtfSpecifiedFolder.getText());
			} else
				subfolder = new Subfolder(dir);
		}
		
		File[] files = dir.listFiles(new FileFilter() {
			public boolean accept(File file) {
				return (file.isDirectory() && !file.getName().equalsIgnoreCase(SUBFOLDER));
			}
		});

		if (jcbOnlyUsFiles.isSelected()) {
			if (dir.getName().matches(".*_files"))
				processSourceDirectory(dir, subfolder);
		} else
			processSourceDirectory(dir, subfolder);
		
		if (jrbTargetSubfolder.isSelected())
			subfolder = new Subfolder(dir);
		
		for (int i = 0; i < files.length; i++) {
			if(!stopButton.isEnabled())
				break;
			
			if (progressIt && i == 0)
				progressBar1.setMaximum(files.length);
			
			try {
				processDirectory(files[i], subfolder, false);
			} catch (Exception exc) {
				exc.printStackTrace();
				AlertDialog.showErrorDialog(exc);
			}
			
			if (progressIt)
				progressBar1.setValue(i + 1);
		}		
	}	
	
	private void processSourceDirectory(File dir, Subfolder subfolder)
			throws Exception {
		final long minSize = Long.parseLong(jtfMinSize.getText())
				* (1024 ^ jcbMinSize.getSelectedIndex());

		File[] files = dir.listFiles(new FileFilter() {
			public boolean accept(File file) {
				boolean result = false;
				String fname = file.getName().toLowerCase();
				for (int i = 0; i < extArray.length; i++) {
					if (fname.endsWith(extArray[i])) {
						result = true;
						break;
					}
				}
				if (result) {
					if (file.length() < minSize)
						result = false;
				}
				if (result) {
					ImageIcon imageIcon = null;
					try {
						imageIcon = new ImageIcon(file.getPath());
						if(!((imageIcon.getIconWidth() >= geometry1 && imageIcon.getIconHeight() >= geometry2)
								|| (imageIcon.getIconWidth() >= geometry2 && imageIcon.getIconHeight() >= geometry1)))
							result = false;
						if (result && jcbPreview.isSelected())
							imageLabel.setIcon(imageIcon);
					} catch (Exception exc) {
						exc.printStackTrace();
					}
				}
				return result;
			}
		});

		if (files == null || files.length == 0)
			return;

		if (subfolder.dir == null) {
			subfolder.dir = new File(subfolder.parent, SUBFOLDER);
			if (subfolder.dir.exists()) {
				if (jcbClearTargetFolder.isSelected()) {
					logLine("Dir " + subfolder.dir.getPath()
							+ " will be cleared");
					File[] children = subfolder.dir.listFiles();
					for (int i = 0; i < children.length; i++)
						children[i].delete();
				}
			} else
				subfolder.dir.mkdir();
		}

		for (int i = 0; i < files.length; i++) {
			File target = getCopyFile(subfolder, files[i]);
			if (target != null) {
				IOUtils.copyFile(files[i], target);
			}							
/*			
			if (target != null) {
				IOUtils.copyFile(files[i], target);
				if (jcbPreview.isSelected()) {
					ImageIcon imageIcon = null;
					try {
						imageIcon = new ImageIcon(target.getPath());
					} catch (Exception exc) {
						exc.printStackTrace();
					}
					if (imageIcon != null)
						imageLabel.setIcon(imageIcon);
				}
			}
*/
		}
	}

	private File getCopyFile(Subfolder subfolder, File source) throws Exception {
		File target = new File(subfolder.dir, source.getName());
		if(target.exists() && !jrbIterateNoRewrite.isSelected()) {
//			Old plcae of the Identification check block 			
			
			String fname = source.getName();
			String ext = "";
			String fileName = fname;
			int dotIdx = fname.lastIndexOf('.');
			if (dotIdx != -1) {
				ext = fname.substring(dotIdx);
				fileName = fname.substring(0, dotIdx);
			}

			int count = 0;
			while (target.exists()) {
//				Identification check block: Start			
				if(jrbIterateNoSkip.isSelected())
					return null;

				if(jrbIterateSizeDiffers.isSelected()) {
					if(source.length() == target.length())
						return null;
				}

				if (jrbIterateCtntDiffers.isSelected()) {
					if (source.length() == target.length()) {
						if (IOUtils.areFilesIdentical(source, target)) {
							logLine("Files: " + source.getPath() + " and " + 
									target.getPath() + " are identical by content");
							return null;
						}
					}
				}			
//				Identification check block: End			
								
				count++;
				String newName = fileName + "-" + String.valueOf(count) + ext;
				target = new File(subfolder.dir, newName);
			}
						
		}
		return target;
	}
	
	/**
	 * This method initializes jtfSpecifiedFolder	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getJtfSpecifiedFolder() {
		if (jtfSpecifiedFolder == null) {
			jtfSpecifiedFolder = new JTextField();
			jtfSpecifiedFolder.setPreferredSize(new java.awt.Dimension(240,20));
			jtfSpecifiedFolder.setEnabled(false);
		}
		return jtfSpecifiedFolder;
	}

	/**
	 * This method initializes jbSpecFolder	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJbSpecFolder() {
		if (jbSpecFolder == null) {
			jbSpecFolder = new JButton();
			jbSpecFolder.setText("...");
			jbSpecFolder.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jbSpecFolder_actionPerformed(e);
				}
			});
			jbSpecFolder.setPreferredSize(new java.awt.Dimension(43,20));
		}
		return jbSpecFolder;
	}
	
	void jbSpecFolder_actionPerformed(java.awt.event.ActionEvent e) {
		File file = Context.dirOpenDialog(this);
		if(file == null)
			return;
		jtfSpecifiedFolder.setText(file.getPath());					
	}
		
	private void logLine(String line) {
		jEditorPane1.setText(jEditorPane1.getText() + line + "\r\n");
	}

	/**
	 * This method initializes rightPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getRightPanel() {
		if (rightPanel == null) {
			imageLabel = new JLabel();
			imageLabel.setPreferredSize(new Dimension(420, 50));
			imageLabel.setName("jLabel");
			imageLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
			imageLabel.setText("Preview");
			rightPanel = new JPanel();
			rightPanel.setLayout(new CardLayout());
			rightPanel.setPreferredSize(new java.awt.Dimension(280,10));
			rightPanel.add(imageLabel, imageLabel.getName());
		}
		return rightPanel;
	}

	/**
	 * This method initializes jcbPreview	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */    
	private JCheckBox getJcbPreview() {
		if (jcbPreview == null) {
			jcbPreview = new JCheckBox();
			jcbPreview.setPreferredSize(new java.awt.Dimension(450,20));
			jcbPreview.setText(Strings.getString("MainFrame.preview"));
		}
		return jcbPreview;
	}

	/**
	 * This method initializes stopButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getStopButton() {
		if (stopButton == null) {
			stopButton = new JButton();
			stopButton.setText("Stop");
			stopButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					stopButton.setEnabled(false);
				}
			});
			stopButton.setPreferredSize(new java.awt.Dimension(60,20));
		}
		return stopButton;
	}

	/**
	 * This method initializes jtfMinGeometry	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getJtfMinGeometry() {
		if (jtfMinGeometry == null) {
			jtfMinGeometry = new JTextField();
			jtfMinGeometry.setPreferredSize(new java.awt.Dimension(200,20));
			jtfMinGeometry.setText("300x200");
		}
		return jtfMinGeometry;
	}
	
	private void applySettings() throws Exception {
		File sf = Context.getSettingsFile();
		StringBuffer sb = new StringBuffer();
		sb.append("sb.finder.srcdir=" + jtfSourceDir.getText().replaceAll("\\\\", "\\\\\\\\") + "\n");
		sb.append("sb.finder.minsize=" + jtfMinSize.getText() + "\n");
		sb.append("sb.finder.minsizedim=" + jcbMinSize.getSelectedIndex()
				+ "\n");
		sb.append("sb.finder.maxsize=" + jtfMaxSize.getText() + "\n");
		sb.append("sb.finder.maxsizedim=" + jcbMaxSize.getSelectedIndex()
				+ "\n");
		
		int targetMode = 0;
		if(jrbTargetSubfolder.isSelected())
			targetMode = 0;			
		else if(jrbRootSubfolder.isSelected())
			targetMode = 1;	
		else if(jrbSpecifedFolder.isSelected())
			targetMode = 2;		
		
		sb.append("sb.finder.targetmode=" + targetMode
				+ "\n");
		sb.append("sb.finder.targetfolder="
				+ jtfSpecifiedFolder.getText().replaceAll("\\\\", "\\\\\\\\")
				+ "\n");
		sb.append("sb.finder.onlyusfiles="
				+ Boolean.toString(jcbOnlyUsFiles.isSelected()) + "\n");
		
		int iterateMode = 0;
		if(jrbIterateYes.isSelected())
			iterateMode = 0;
		else if (jrbIterateSizeDiffers.isSelected())
			iterateMode = 1;
		else if (jrbIterateCtntDiffers.isSelected())
			iterateMode = 2;
		else if (jrbIterateNoSkip.isSelected())
			iterateMode = 3;
		else if (jrbIterateNoRewrite.isSelected())
			iterateMode = 4;			
		
		sb.append("sb.finder.iteratemode=" + iterateMode
				+ "\n");
		sb.append("sb.finder.cleartarget="
				+ Boolean.toString(jcbClearTargetFolder.isSelected()) + "\n");
		sb.append("sb.finder.preview="
				+ Boolean.toString(jcbPreview.isSelected()) + "\n");
		sb.append("sb.finder.mingeometry=" + jtfMinGeometry.getText()
				+ "\n");
		ByteArrayInputStream bais = new ByteArrayInputStream(sb.toString()
				.getBytes("UTF-8"));
		IOUtils.writeFile(bais, sf);
	}
	
	private void loadSettings() throws Exception {
		Context.loadSettings();
		String value = Context.getProperty("sb.finder.srcdir");
		if (value != null) {
			jtfSourceDir.setText(value);
			Context.setFIleDlgOpenPath(value);
		}
		value = Context.getProperty("sb.finder.minsize");
		if (value != null)
			jtfMinSize.setText(value);
		value = Context.getProperty("sb.finder.minsizedim");
		if (value != null)
			jcbMinSize.setSelectedIndex(Integer.parseInt(value));
		value = Context.getProperty("sb.finder.maxsize");
		if (value != null)
			jtfMaxSize.setText(value);
		value = Context.getProperty("sb.finder.maxsizedim");
		if (value != null)
			jcbMaxSize.setSelectedIndex(Integer.parseInt(value));
		value = Context.getProperty("sb.finder.targetmode");
		if (value != null) {
			int targetMode = Integer.parseInt(value);
			switch (targetMode) {
			case 0:
				jrbTargetSubfolder.setSelected(true);
				break;
			case 1:
				jrbRootSubfolder.setSelected(true);
				break;
			case 2:
				jrbSpecifedFolder.setSelected(true);
				break;
			}
		}
		value = Context.getProperty("sb.finder.targetfolder");
		if (value != null)
			jtfSpecifiedFolder.setText(value);
		value = Context.getProperty("sb.finder.onlyusfiles");
		if (value != null)
			jcbOnlyUsFiles.setSelected(Boolean.parseBoolean(value));
		value = Context.getProperty("sb.finder.iteratemode");
		if (value != null) {
			int iterateMode = Integer.parseInt(value);
			switch (iterateMode) {
			case 0:
				jrbIterateYes.setSelected(true);
				break;
			case 1:
				jrbIterateSizeDiffers.setSelected(true);
				break;
			case 2:
				jrbIterateCtntDiffers.setSelected(true);
				break;
			case 3:
				jrbIterateNoSkip.setSelected(true);
				break;
			case 4:
				jrbIterateNoRewrite.setSelected(true);
				break;
			}
		}
		value = Context.getProperty("sb.finder.cleartarget");
		if (value != null)
			jcbClearTargetFolder.setSelected(Boolean.parseBoolean(value));
		value = Context.getProperty("sb.finder.preview");
		if (value != null)
			jcbPreview.setSelected(Boolean.parseBoolean(value));
		value = Context.getProperty("sb.finder.mingeometry");
		if (value != null)
			jtfMinGeometry.setText(value);
	}
	
}  // @jve:decl-index=0:visual-constraint="10,10"
